var celery = require('celery'),
service = require('./service');


function unableToFetch()
{
    return console.log('Unable to fetch'.red);
}




celery.on({

     /**
      * list apps 
      */
     
     'list': function()
     {
        service.request('/garden/apps', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            response.result.forEach(function(app)
            {
                if(!app.nodes.length)
                {
                    console.log(app.name.blue +' - not running'.red);
                }
                else
                {
                    for(var i = app.nodes.length; i--;)
                    {
                        var node = app.nodes[i];
                        
                        console.log(app.name.blue + (' - '+(node.statename)+' '+node.description).green);
                    }
                }
            });
        });
     },
    
    
    
    /**
     */
     
    ':appName history': function(data)
    {
        
        service.request('/garden/apps/'+data.appName+'/checkout/history', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            if(!response.result.length) return console.log('No commits');
            
            response.result.forEach(function(checkout)
            {
                console.log(data.appName+'@'+checkout.package.version +' - '+new Date(checkout.lastUsed));
            });
        });
    },
    
    /**
     */
     
    ':appName rollback': function(data)
    {
        
        service.request('/garden/apps/'+data.appName+'/rollback', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            console.log('Rolling back to previous version');
        });
    },
    
    /**
     * returns the app logs
     */
     
    ':appName logs': function(data)
    {
        console.log("LOGS");
    },
    
    /**
     */
     
    ':appName processes': function(data)
    {
    },
    
    /**
     */
     
    ':appName checkout :repository': function(data)
    {
        celery.emit(data.appName+' checkout', { repository: data.repository });
    },
    
    /**
     */
     
    ':appName checkout': function(data)
    {
        var loading = celery.loading('Checking out '+data.appName+': ');
        
        if(data.repository) data.repository = decodeURIComponent(data.repository);
        
        service.request({ path: '/garden/apps/'+data.appName+'/checkout', method: 'post', data: { repository: data.repository }}, function(err, result)
        {
            loading.done(!result.errors);
        });
    },
    
    /**
     * Starts interactive console with app - stdin
     */
     
    ':appName console': function()
    {
        console.log("TODO");
    },
    
    /**
     */
     
    ':appName logs tail': function()
    {
        console.log("TAIL LOGS");
    },
    
    /**
     * starts an app
     */
     
    'start :appName': function(data)
    {
        service.request('/garden/apps/'+data.appName+'/start', function(err, result){ });
    },
    
    
    /**
     * stops an app
     */
     
    'stop :appName': function(data)
    {
        service.request('/garden/apps/'+data.appName+'/stop', function(err, result){ });
    },
    
    /** 
     * restart target app
     */
     
    'restart :appName': function()
    {
        service.request('/garden/apps/'+data.appName+'/restart', function(err, result){ });
    },
    
    
    
    /**
     * creates a new app
     */
     
    'create :appName': function(data)
    {
        service.request({ path: '/garden/apps/', method: 'post', data: { name: data.appName } }, function(err, result)
        {
            if(result.errors) return console.log('Unable to create app'.red);
            
            console.log('Successfuly created app');
        });
    },
    
    /**
     */
     
    'remove :appName': function(data)
    {
        service.request({ path: '/garden/apps/' + data.appName+'/remove' }, function(err, result)
        {
            if(result.errors) return console.log('Unable to delete'.red);
            
            console.log('Successfuly deleted app');
        })
    },
    
    
    /**
     * restart all
     */
     
    'restart': function()
    {
        console.log("TODO");
    },
    /**
     */
     
    'help': function()
    {
        //celery.openHelp(__dirname + '/help.txt');
    },
    
    /**
     * shovel version
     */
     
    'version': function()
    {
    },
    
    /**
     */
     
    'signup': function()
    {
        celery.auth(function(user, pass)
        {
            console.log(user);
            console.log(pass);
        });
    },
    
    /**
     */
     
    'logout': function()
    {
        console.log("logged out".grey);
        auth();
    }
});


function signup(user, pass)
{
    service.signup(function(err, result)
    {
        if(err || result.errors)
        {
            console.error('Unable to signup'.red);
        }
        else
        {
            loggedIn();
        }
    });
}

function loggedIn()
{
    console.ok('logged in as %s'.green, service.user);
}


console.log('First time logging in? enter in new credentials and click "y"');


function auth()
{
    celery.auth(function(user, pass)
    {
        service.setAuth(user, pass);
        
        
        service.validate(function(err, result)
        {
            if(err) return console.log('Unable to validate credentials'.red); 
            
            if(result.errors)
            {
                celery.confirm('Auth credits are incorrect. Would you like to signup?', function(yes)
                {
                    if(yes) return signup(user, pass);
                    
                    auth();
                });
            }
            else
            {
                loggedIn();
            }
        });
    });
}


auth();




celery.open();

celery.parse(process.argv);